@@ -93,6 +93,11 @@ const (
 	Duration   // time.Duration
 	Ext        // extension
 	JsonNumber // json.Number
+	AInt64
+	AUint64
+	AInt32
+	AUint32
+	ABool
 
 	IDENT // IDENT means an unrecognized identifier
 )
@@ -125,6 +130,11 @@ var primitives = map[string]Primitive{
 	"time.Duration":  Duration,
 	"msgp.Extension": Ext,
 	"json.Number":    JsonNumber,
+	"atomic.Int64":   AInt64,
+	"atomic.Uint64":  AUint64,
+	"atomic.Int32":   AInt32,
+	"atomic.Uint32":  AUint32,
+	"atomic.Bool":    ABool,
 }
 
 // types built into the library
@@ -139,14 +149,24 @@ var builtins = map[string]struct{}{
 type common struct {
 	vname, alias string
 	ptrRcv       bool
+	typeParams   GenericTypeParams // Generic type parameters, e.g., "[T]"
 }
 
-func (c *common) SetVarname(s string) { c.vname = s }
-func (c *common) Varname() string     { return c.vname }
-func (c *common) Alias(typ string)    { c.alias = typ }
-func (c *common) hidden()             {}
-func (c *common) AllowNil() bool      { return false }
-func (c *common) SetIsAllowNil(bool)  {}
+// GenericTypeParams is a struct that contains the generic type parameters for an element.
+type GenericTypeParams struct {
+	TypeParams   string
+	ToPointerMap map[string]string
+	isPtr        bool
+}
+
+func (c *common) SetVarname(s string)                { c.vname = s }
+func (c *common) Varname() string                    { return c.vname }
+func (c *common) Alias(typ string)                   { c.alias = typ }
+func (c *common) hidden()                            {}
+func (c *common) AllowNil() bool                     { return false }
+func (c *common) SetIsAllowNil(bool)                 {}
+func (c *common) SetTypeParams(tp GenericTypeParams) { c.typeParams = tp }
+func (c *common) TypeParams() GenericTypeParams      { return c.typeParams }
 func (c *common) AlwaysPtr(set *bool) bool {
 	if c != nil && set != nil {
 		c.ptrRcv = *set
@@ -218,6 +238,12 @@ type Elem interface {
 	// Note that this is NOT used by the `omitzero` feature.
 	IfZeroExpr() string
 
+	// SetTypeParams sets the generic type parameters for this element
+	SetTypeParams(tp GenericTypeParams)
+
+	// TypeParams returns the generic type parameters for this element
+	TypeParams() GenericTypeParams
+
 	hidden()
 }
 
@@ -707,6 +733,17 @@ func (s *BaseElem) BaseType() string {
 		return "time.Duration"
 	case JsonNumber:
 		return "json.Number"
+	case AInt64:
+		return "atomic.Int64"
+	case AUint64:
+		return "atomic.Uint64"
+	case AInt32:
+		return "atomic.Int32"
+	case AUint32:
+		return "atomic.Uint32"
+	case ABool:
+		return "atomic.Bool"
+
 	case Ext:
 		return "msgp.Extension"
 
@@ -787,6 +824,12 @@ func (s *BaseElem) ZeroExpr() string {
 
 // IfZeroExpr returns the expression to compare to zero/empty.
 func (s *BaseElem) IfZeroExpr() string {
+	switch s.Value {
+	case AInt64, AUint64, AInt32, AUint32:
+		return fmt.Sprintf("%s.Load() == 0", s.Varname())
+	case ABool:
+		return fmt.Sprintf("!%s.Load()", s.Varname())
+	}
 	z := s.ZeroExpr()
 	if z == "" {
 		return ""
@@ -842,6 +885,16 @@ func (k Primitive) String() string {
 		return "Extension"
 	case JsonNumber:
 		return "json.Number"
+	case AInt64:
+		return "atomic.Int64"
+	case AUint64:
+		return "atomic.Uint64"
+	case AInt32:
+		return "atomic.Int32"
+	case AUint32:
+		return "atomic.Uint32"
+	case ABool:
+		return "atomic.Bool"
 	case IDENT:
 		return "Ident"
 	default:
